from __future__ import print_function
from os import getcwd
import argparse
from treesource import generate_tree, render


if __name__ == "__main__":

    # argument parses
    parser = argparse.ArgumentParser(
        prog='python -m treesource',
        description='Source file-trees generator.')
    parser.add_argument('-u', dest='use_unicode', action='store_const',
                        const=True, default=False,
                        help='use special unicode symbols as icons')
    parser.add_argument('-a', dest='show_all', action='store_const',
                        const=True, default=False,
                        help='show all files and folder, not only documented ones')
    parser.add_argument('-r', '--root', metavar='PATH', type=str,
                        default=getcwd(), help='the root directory of the tree')

    parser.add_argument('-f', '--format', metavar='FORMAT', type=str,
                        default='txt', help='the rendering format [txt|md]')
    args = parser.parse_args()

    # generate the tree
    tree = generate_tree(args.root, keep_undocumented=args.show_all)

    # render the tree
    if args.format.upper() in ['MD', 'MARKDOWN']:
        rendered = render.as_markdown(tree, use_unicode=args.use_unicode)
    elif args.format.upper() in ['TXT', 'TEXT', 'ASCII']:
        rendered = render.as_text(tree, use_unicode=args.use_unicode)
    else:
        print("ERROR: Unrecognised format option: {}".format(args.format))
        rendered = render.as_text(tree, use_unicode=args.use_unicode)

    print(rendered)



import os
import re
from anytree import AnyNode, PostOrderIter


def get_doc_string(path, first_lines=30):
    """look for the doctree docstring in the first lines of the file in path"""

    docstring = None

    try:
        f = open(path, 'r')
    except Exception:
        return None

    try:
        for _ in range(first_lines):
            # read the file line by line and search the docstring
            line = f.readline()
            m = re.match(r"^.*?\[treesource\]\s*(.*)$", line)
            if m:
                docstring = m.groups()[0]
                break
    except UnicodeDecodeError:
        # probably a binary file
        docstring = None
    except Exception:
        docstring = None
    finally:
        f.close()

    return docstring


def walk2tree(walk_lsit, prune_undocumented):
    """transform a filtered folder list into a tree."""
    # create root node
    root = AnyNode(id='root', name='.', doc=None,
                   node_type='root', parent=None)
    # add root node's files
    for f in walk_lsit[0]['files']:
        AnyNode(
            id=f['filename'],
            name=f['filename'],
            parent=root,
            doc=f['docstring'],
            node_type='file'
        )

    nodes = {'root': root}

    # add other nodes
    for entry in walk_lsit[1:]:
        dirname = entry['dirname']
        parent_name = entry['parent']
        parent = nodes[parent_name]
        newnode_name = '/'.join([parent_name, dirname])
        node_type = 'dir'

        newnode = AnyNode(
            id=newnode_name, parent=parent,
            name=dirname, doc=entry['dirdoc'], node_type=node_type
        )
        nodes[newnode_name] = newnode

        for f in entry['files']:
            AnyNode(
                id=f['filename'],
                name=f['filename'],
                parent=newnode,
                doc=f['docstring'],
                node_type='file'
            )

    # prune undocumented nodes
    if prune_undocumented:
        for node in PostOrderIter(root):
            if node.doc == None and len(node.children) == 0:
                node.parent = None

    return root


def generate_tree(startpath, keep_undocumented=False):
    """Generate a file-tree object from the current working directory.

    keep_undocumented: keep undocumented files and directories in the tree

    Returns an anytree object

    """
    walked = []

    # iterate over the os.walk result
    for root, _, files in os.walk(startpath):
        # directory level
        level = root.replace(startpath, '').count(os.sep)

        # directory name
        dirname = os.path.basename(root.replace(startpath, ''))

        documented_files = []
        for filename in files:
            # look for the docstring in the file
            docstring = get_doc_string(os.path.join(root, filename))
            # speed up by not inserting undocumented diles
            if keep_undocumented or (docstring is not None):
                documented_files.append(dict(
                    filename=filename,
                    docstring=docstring)
                )

        # get folder doc
        dirdoc = None
        docfile = os.path.join(root, 'treesource.txt')
        if os.path.exists(docfile):
            with open(docfile, 'r') as f:
                dirdoc = f.readline()

        walked.append(dict(
            parent='root' +
            '/'.join((root.replace(startpath, '').split(os.sep)[:-1])),
            level=level,
            dirname=dirname,
            dirdoc=dirdoc,
            files=documented_files)
        )

    root = walk2tree(walked, prune_undocumented=not keep_undocumented)
    return root



from anytree import RenderTree
from .sorting import directory_first_alphabetical


def render_tree(
        tree, folder_icon, file_icon,
        doc_folder_format, no_doc_folder_format,
        doc_file_format, no_doc_file_format):
    """Render the doctree as text

    tree: the file-tree
    folder and file_icon : chars to use before file/folder names
    formats: format templates for the file/folder names

    # The rendering FORMATS use special tokens:
    pre: the ASCII chars that represent the tree
    icon: an icon displayed is use_unicode is true
    name: the file/directory name
    doc: the documentation string

    # EXAMPLE:
    # folder that has a documentation
    #    doc_folder_format = "{pre}{icon}{name}\\ ({doc})"
    # folder without documentation
    #    no_doc_folder_format = "{pre}{icon}{name}\\"   
    # file that has a documentation
    #    doc_file_format = "{pre}{icon}{name} ({doc})"
    # file without documentation
    #    no_doc_file_format = "{pre}{icon}{name}"
    """

    rendered = ""

    for pre, _, node in RenderTree(tree, childiter=directory_first_alphabetical):
        # directories
        if node.node_type == 'dir':
            if node.doc is None:  # no doc
                rendered += no_doc_folder_format.format(
                    pre=pre, icon=folder_icon, name=node.name, doc=''
                ) + "\n"
            else:
                rendered += doc_folder_format.format(
                    pre=pre, icon=folder_icon, name=node.name, doc=node.doc
                ) + "\n"
        # files
        elif node.node_type == 'file':
            if node.doc is None:  # no doc
                rendered += no_doc_file_format.format(
                    pre=pre, icon='', name=node.name, doc=''
                ) + "\n"
            else:
                rendered += doc_file_format.format(
                    pre=pre, icon=file_icon, name=node.name, doc=node.doc
                ) + "\n"
        else:
            rendered += no_doc_file_format.format(
                pre=pre, icon='', name=node.name, doc=''
            ) + "\n"

    return rendered



from . import engine


def as_text(tree, use_unicode=False):
    """render the doctree as text"""
    # The rendering FORMATS
    # pre: the ASCII chars that represent the tree
    # icon: an icon displayed is use_unicode is true
    # name: the file/directory name
    # doc: the documentation string

    # folder that has a documentation
    doc_folder_format = "{pre}{icon}{name}\\ ({doc})"
    # folder without documentation
    no_doc_folder_format = "{pre}{icon}{name}\\"
    # file that has a documentation
    doc_file_format = "{pre}{icon}{name} ({doc})"
    # file without documentation
    no_doc_file_format = "{pre}{icon}{name}"

    rendered = ""

    # unicode icons
    if use_unicode:
        folder_icon = "🗀 "
        file_icon = "🗋 "
    else:
        folder_icon = ""
        file_icon = ""

    rendered = engine.render_tree(
        tree,
        folder_icon, file_icon,
        doc_folder_format, no_doc_folder_format,
        doc_file_format, no_doc_file_format
    )

    return rendered


def as_markdown(tree, use_unicode=True):
    """render the doctree as markdown"""

    # The rendering FORMATS
    # pre: the ASCII chars that represent the tree
    # icon: an icon displayed is use_unicode is true
    # name: the file/directory name
    # doc: the documentation string

    # folder that has a documentation
    doc_folder_format = "`{pre}{icon}`**`{name}\\`** _{doc}_\\"
    # folder without documentation
    no_doc_folder_format = "`{pre}{icon}`**`{name}\\`**\\"
    # file that has a documentation
    doc_file_format = "`{pre}{icon}{name}` _{doc}_\\"
    # file without documentation
    no_doc_file_format = "`{pre}{icon}{name}`\\"

    rendered = ""

    # unicode icons
    if use_unicode:
        folder_icon = "🗀 "
        file_icon = "🗋 "
    else:
        folder_icon = ""
        file_icon = ""

    rendered = engine.render_tree(
        tree,
        folder_icon, file_icon,
        doc_folder_format, no_doc_folder_format,
        doc_file_format, no_doc_file_format
    )

    return rendered



def select_by_type(node_type, nodes):
    return filter(lambda node: node.node_type==node_type, nodes)

def sort_by_name(nodes):
    return sorted(nodes, key=lambda node: node.name)

def directory_first_alphabetical(nodes):
    """sort in alphabetical order, first directories then files"""
    dirs = select_by_type('dir', nodes)
    files = select_by_type('file', nodes)
    return sort_by_name(dirs) + sort_by_name(files)


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(name='treesource',
      version='0.0.3',
      description=' commented source file-trees generator ',
      author='Marco Pascucci',
      long_description=long_description,
      long_description_content_type="text/markdown",
      author_email='marpas.paris@gmail.com',
      url='https://github.com/mpascucci/treesource-doc',
      packages=['treesource', 'treesource.render'],
      install_requires=['anytree'],
      classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
    ]
)